% objdalgl - objective function of DAL with grouped L1 regularization
%
% Copyright(c) 2009 Ryota Tomioka
% This software is distributed under the MIT license. See license.txt

function varargout=objdalgl(aa, fnc, ww, A, AT, lambda, eta, solver)

[ns,nc] = size(ww);
n = ns*nc;

vv    = -ww/eta;
vv(:) = vv(:)+AT(aa);
nm    = sqrt(sum(vv.^2));
I     = find(nm>lambda);


[floss, gloss, hloss]=fnc.d(aa, fnc.args{:});

m = size(gloss,1);

fval = floss+0.5*eta*sum((nm(I)-lambda).^2);

varargout{1}=fval;

if nargout>1
  I2 = zeros(ns,nc); I2(:,I)=1; I2=find(I2);
  vss=gl_softth(vv(:,I),lambda);
  dd = sparse(I2,1,-eta*vss(:),n,1);
  varargout{2} = gloss-A(dd);
  if nargout>2
    switch(solver)
      case 'cg'
       % prec = spdiag(hloss+eta*sum(AF.^2,2));
       prec = hloss;
       ff   = zeros(1,length(I));
       vn   = zeros(ns,length(I));
       for kk=1:length(I)
         jj=I(kk);
         vn(:,kk)=vv(:,jj)/nm(jj);
         ff(kk)=sqrt(1-lambda/nm(jj));
         iblk=((1:ns)+ns*(jj-1))';

         F1 = sparse(iblk,1:ns,ff(kk)*ones(ns,1),ns*nc,ns);
         F2 = sparse(iblk,ones(ns,1),(1-ff(kk))*vn(:,kk),ns*nc,1);
         prec = prec + eta*(sum(A(F1).^2,2)+A(F2).^2);
       end
       prec = spdiag(prec);
       varargout{3} = struct('ns',ns,'nc',nc,'hloss',hloss,...
                             'I',I,'vn',vn,'ff',ff,'prec',prec);
     case 'nt'
      AF = zeros(m, ns*length(I));
      for kk=1:length(I)
        jj=I(kk);
        vn=vv(:,jj)/nm(jj);
        ff=sqrt(1-lambda/nm(jj));
        iblk=((1:ns)+ns*(jj-1))';
        
        F1 = sparse(iblk,1:ns,ff*ones(ns,1),ns*nc,ns);
        F2 = sparse(iblk,ones(ns,1),(1-ff)*vn,ns*nc,1);
        AF(:,(1:ns)+ns*(kk-1))=A(F1)+A(F2)*vn';
      end
       varargout{3} = spdiag(hloss)+ eta*AF*AF';
     case 'ntsv'
      H=diag(hloss);
      for kk=1:length(I)
        jj=I(kk);
        vn=vv(:,jj)/nm(jj);
        ff=sqrt(1-lambda/nm(jj));
        iblk=((1:ns)+ns*(jj-1))';

        F1 = sparse(iblk,1:ns,ff*ones(ns,1),ns*nc,ns);
        F2 = sparse(iblk,ones(ns,1),(1-ff)*vn,ns*nc,1);
        AF=A(F1)+A(F2)*vn';
        H = H+eta*AF*AF';
      end
      varargout{3} = H;
    end
  end
end

